import json
from mercuryclient.types.bank_account_verify.request import AccountVerifyRequest


class AccountVerifyMixin:
    """
    Mixin for bank account verification
    """

    def verify_bank_account(
        self, request_obj: AccountVerifyRequest, provider: str, profile: str
    ):
        """
        bank account verification using the mercury service

        :param request_obj: AccountVerifyRequest
        :param provider: cashfree
        :param profile: An existing profile in Mercury. The profile has to correspond
        to the provider.
        :return: (request_id, data)
        """
        api = "api/v1/account_verify/"

        request_dict = json.loads(request_obj.json(exclude_unset=True))
        request_dict["profile"] = profile
        request_dict["provider"] = provider

        request_id, r = self._post_json_http_request(
            path=api, data=request_dict, send_request_id=True, add_bearer_token=True
        )

        try:
            response_json = r.json()
        except Exception:
            response_json = {}

        if r.status_code == 201:
            return {"request_id": request_id, "data": response_json}

        raise Exception(
            "Error while sending account verify request. Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )



from unittest import TestCase, mock
from mercuryclient.api import MercuryApi


class RechargeMixinTest(TestCase):
    def setUp(self):

        self.client = MercuryApi(
            {
                "username": "username",
                "password": "password",
                "url": "https://mercury-dev.esthenos.in",
            }
        )

        self.request_dict = {"operator": 1, "number": 9876543210, "amount": 10}

        self.expected_dict = {
            "operator": 1,
            "number": 9876543210,
            "amount": 10,
            "profile": "some_profile",
        }
        self.post_api_mock = mock.patch(
            "mercuryclient.api.MercuryApi._post_json_http_request"
        ).start()
        self.addCleanup(self.post_api_mock.stop)

    def test_request_recharge(self):

        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        self.client.make_recharge(self.request_dict, "some_profile")
        self.post_api_mock.assert_called_with(
            path="api/v1/mobile_recharge/recharge/",
            data=self.expected_dict,
            send_request_id=True,
            add_bearer_token=True,
        )

    def test_request_exception_raised_if_status_code_error(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 401
        self.post_api_mock.return_value = ("random_string", mock_response)
        with self.assertRaises(Exception):
            self.client.make_recharge(self.request_dict, "some_profile")

    def test_request_api_succeeds_if_status_code_success(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_id = self.client.make_recharge(self.request_dict, "some_profile")
        self.assertEqual(request_id["request_id"], "random_string")



from datetime import date
from unittest import TestCase, mock

from mercuryclient.api import MercuryApi


class RekognitionMixinTest(TestCase):
    def setUp(self):
        self.client = MercuryApi(
            {
                "username": "username",
                "password": "password",
                "url": "https://mercury-dev.esthenos.in",
            }
        )

        self.post_api_mock = mock.patch(
            "mercuryclient.api.MercuryApi._post_json_http_request"
        ).start()
        self.addCleanup(self.post_api_mock.stop)
        self.get_api_mock = mock.patch(
            "mercuryclient.api.MercuryApi._get_json_http_request"
        ).start()
        self.addCleanup(self.get_api_mock.stop)
        self.request_data = dict(
            profile="test",
            provider="test",
            token="test_token",
            liveness_prefix="test_prefix",
        )

    def test_create_session_id(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)

        self.client.generate_liveness_session_id(**self.request_data)

        self.post_api_mock.assert_called_with(
            "api/v1/rekognition/create/session",
            data={
                "profile": "test",
                "provider": "test",
                "token": "test_token",
                "liveness_prefix": "test_prefix",
            },
            send_request_id=True,
            add_bearer_token=True,
        )

    def test_request_exception_raised_if_status_code_error(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 401
        self.post_api_mock.return_value = ("random_string", mock_response)

        with self.assertRaises(Exception):
            self.client.generate_liveness_session_id(**self.request_data)

    def test_request_api_succeeds_if_status_code_success(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)

        response = self.client.generate_liveness_session_id(**self.request_data)
        self.assertEqual(response[0], "random_string")



from datetime import date
from typing import List
from typing import Optional

from pydantic import BaseModel
from pydantic import Field
from pydantic import root_validator
from pydantic import validator

from ..common import Pincode
from ..validators import normalize_str
from .enums import EmploymentStatus
from .enums import EnquiryReason
from .enums import FinancePurpose
from .enums import Gender
from .enums import MaritalStatus
from .enums import PhoneNumberType
from .enums import State


class PhoneNumber(BaseModel):
    phone_number: str = Field(max_length=20)
    extension: Optional[str] = Field(max_length=10)
    number_type: Optional[PhoneNumberType]

    _normalize_num = validator("phone_number", allow_reuse=True, pre=True)(
        normalize_str
    )


class Identity(BaseModel):
    id_number: str
    issue_date: Optional[date]
    expiration_date: Optional[date]

    _normalize_str = validator("id_number", allow_reuse=True, pre=True)(normalize_str)

    @root_validator
    def check_issue__before_expiration(cls, values: dict):
        if values.get("issue_date") and values.get("expiration_date"):
            assert (
                values["issue_date"] < values["expiration_date"]
            ), "issue date must be before expiration date"

        return values


class Address(BaseModel):
    address_line_1: str = Field(max_length=200)
    address_line_2: Optional[str] = Field(max_length=200)
    address_line_3: Optional[str] = Field(max_length=200)
    city: str = Field(max_length=50)
    landmark: Optional[str] = Field(max_length=40)
    state: State
    pincode: Pincode

    _normalize_strs = validator("address_line_1", "city", allow_reuse=True, pre=True)(
        normalize_str
    )


class Applicant(BaseModel):
    first_name: str = Field(max_length=40)
    middle_name_1: Optional[str] = Field(max_length=40)
    middle_name_2: Optional[str] = Field(max_length=40)
    middle_name_3: Optional[str] = Field(max_length=40)
    last_name: str = Field(max_length=60)
    gender: Gender
    date_of_birth: date
    email: Optional[str] = Field(max_length=80)
    phone_number: Optional[PhoneNumber]
    mobile_number: Optional[str] = Field(max_length=20)
    income: Optional[str] = Field(max_length=15)
    marital_status: Optional[MaritalStatus]
    employment_status: Optional[EmploymentStatus]
    time_with_employer: Optional[str] = Field(max_length=3)
    number_of_major_credit_cards_held: Optional[int] = Field(lt=1000)
    pan: Optional[Identity]
    passport: Optional[Identity]
    voter_id: Optional[Identity]
    drivers_licence: Optional[Identity]
    ration_card: Optional[Identity]
    uid: Optional[Identity]
    addresses: List[Address] = Field(min_items=1, max_items=2)

    _normalize_strs = validator("first_name", "last_name", allow_reuse=True, pre=True)(
        normalize_str
    )

    @root_validator
    def check_min_one_identity(cls, values: dict):
        identities = list()
        if values.get("phone_number"):
            identities.append(values["phone_number"].phone_number)

        identity_keys = ["pan", "passport", "voter_id"]
        for key in identity_keys:
            if values.get(key):
                identities.append(values[key].id_number)

        assert any(
            identities
        ), "at least one of the following is required: phone_number, pan, passport, voter_id"

        return values


class ExperianRequest(BaseModel):
    ft_reference_number: Optional[str] = Field(max_length=6)
    customer_reference_number: Optional[str] = Field(max_length=12)
    enquiry_reason: EnquiryReason
    finance_purpose: FinancePurpose
    amount_financed: int
    duration_of_agreement: int = Field(lt=1000)
    applicant: Applicant



from enum import Enum


class IDTypes(Enum):
    PAN_CARD = "PAN_CARD"
    DRIVING_LICENSE = "DRIVING_LICENSE"
    VOTER_ID = "VOTER_ID"
    PASSPORT = "PASSPORT"


class Gender(Enum):
    MALE = "MALE"
    FEMALE = "FEMALE"
    TRANSGENDER = "TRANSGENDER"


class PassportTypes(Enum):
    PERSONAL = "PERSONAL"
    SERVICE = "SERVICE"
    DIPLOMATIC = "DIPLOMATIC"


class Countries(Enum):
    INDIA = "INDIA"



class NameMatchMixin:
    """
    Mixin for name matching
    """

    def name_match(
        self,
        provider: str,
        profile: str,
        primary_name: str,
        secondary_name: str,
        remove_salutations: bool = True,
    ):
        """
        Name Matching using the mercury service

        Args:
            provider (str): service provider
            profile (str): mercury profile
            primary_name (str): name for comparison
            secondary_name (str): name to be compared with
            remove_salutations (bool, optional): Flag to remove salutation. Defaults to True.

        Raises:
            Exception: if Failed to ret response form mercury

        Returns:
            dict[str, dict]: mercury request id,name match result
        """
        api = "api/v1/name/match"

        request_dict = dict(
            profile=profile,
            provider=provider,
            primary_name=primary_name,
            secondary_name=secondary_name,
            remove_salutations=remove_salutations,
        )

        request_id, r = self._post_json_http_request(
            path=api, data=request_dict, send_request_id=True, add_bearer_token=True
        )

        try:
            response_json = r.json()
        except Exception:
            response_json = {}

        if r.status_code == 200:
            return request_id, response_json

        raise Exception(
            "Error while sending name match request. Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )



from typing import Optional
from mercuryclient.types.bbps.enums import ComplaintType, ShopType
from mercuryclient.types.common import Pincode
from pydantic import BaseModel
from pydantic import Field


class AgentRequest(BaseModel):
    name: str = Field(max_length=50)
    contactperson: str = Field(max_length=50)
    mobileNumber: str = Field(max_length=10, min_length=10)
    agentshopname: str = Field(max_length=50)
    businesstype: ShopType
    address1: str = Field(max_length=50)
    address2: str = Field(max_length=50)
    state: str = Field()
    city: str = Field()
    pincode: Pincode
    latitude: str = Field(max_length=50)
    longitude: str = Field(max_length=50)
    email: str = Field(max_length=50)


class GetAmountRequest(BaseModel):

    billerId: str = Field(max_length=50)
    mobileNumber: str = Field(max_length=10, min_length=10)
    crno: str = Field(max_length=50)
    agentId: str = Field(max_length=50)


class BillPaymentRequestTransid(BaseModel):

    billAmount: str = Field(max_length=50)
    transid: str = Field(max_length=50)
    mobileNumber: str = Field(max_length=10, min_length=10)
    agentId: str = Field(max_length=50)
    BillerCategory: str = Field(max_length=50)
    billerId: str = Field(max_length=50)
    crno: str = Field(max_length=50)


class BillPaymentRequest(BaseModel):

    billAmount: str = Field(max_length=50)
    mobileNumber: str = Field(max_length=10, min_length=10)
    agentId: str = Field(max_length=50)
    BillerCategory: str = Field(max_length=50)
    billerId: str = Field(max_length=50)
    crno: str = Field(max_length=50)


class TxnComplaintRequest(BaseModel):
    transactionRefId: str = Field(max_length=50)
    mobileNumber: str = Field(max_length=10, min_length=10)
    reason: str = Field(max_length=70)
    description: Optional[str] = Field(max_length=70)


class ServiceComplaintRequest(BaseModel):
    billerId: str = Field(max_length=50)
    mobileNumber: str = Field(max_length=10, min_length=10)
    description: str = Field(max_length=70)
    reason: str = Field(max_length=50)
    type: str = Field(max_length=50)


class ComplaintStatusRequest(BaseModel):
    complaintType: ComplaintType
    complaintId: str = Field(max_length=50)



import re
from .validators import normalize_str


class Pincode(str):
    """
    Type for Indian pincodes. Verifies that pincodes are all numbers, exactly 6
    characters and start with non-zero
    """

    @classmethod
    def __get_validators__(cls):
        yield cls.validate

    @classmethod
    def validate(cls, value):
        if not isinstance(value, str):
            raise TypeError("string required")
        if re.fullmatch(r"^[1-9][0-9]{5}$", value) is None:
            raise ValueError("invalid pincode format")

        return cls(value)


class NormalizedString(str):
    """
    Type for normalizing strings by stripping and removing newline characters. The type
    also checks the string after normalising and fails if Falsy. WARNING: using this
    type in conjunction with Field function seems to remove its validation. For now,
    only use this type directly if not using length or regex validation with Field(...)
    """

    @classmethod
    def __get_validators__(cls):
        yield cls.validate

    @classmethod
    def validate(cls, value):
        if not isinstance(value, str):
            raise TypeError("string required")

        return cls(normalize_str(value))



from enum import Enum


class PhoneNumberType(Enum):
    """
    Enum for Equifax phone number types
    """

    HOME = "HOME"
    OFFICE = "OFFICE"
    MOBILE = "MOBILE"


class State(Enum):
    """
    Enum for Equifax States and state codes
    """

    ANDAMAN_AND_NICOBAR_ISLANDS = "ANDAMAN_AND_NICOBAR_ISLANDS"
    ANDHRA_PRADESH = "ANDHRA_PRADESH"
    ARUNACHAL_PRADESH = "ARUNACHAL_PRADESH"
    ASSAM = "ASSAM"
    BIHAR = "BIHAR"
    CHATTISGARH = "CHATTISGARH"
    CHANDIGARH = "CHANDIGARH"
    DAMAN_AND_DIU = "DAMAN_AND_DIU"
    DELHI = "DELHI"
    DADRA_AND_NAGAR_HAVELI = "DADRA_AND_NAGAR_HAVELI"
    GOA = "GOA"
    GUJARAT = "GUJARAT"
    HIMACHAL_PRADESH = "HIMACHAL_PRADESH"
    HARYANA = "HARYANA"
    JHARKHAND = "JHARKHAND"
    JAMMU_AND_KASHMIR = "JAMMU_AND_KASHMIR"
    KARNATAKA = "KARNATAKA"
    KERALA = "KERALA"
    LAKSHADWEEP = "LAKSHADWEEP"
    MAHARASHTRA = "MAHARASHTRA"
    MEGHALAYA = "MEGHALAYA"
    MANIPUR = "MANIPUR"
    MADHYA_PRADESH = "MADHYA_PRADESH"
    MIZORAM = "MIZORAM"
    NAGALAND = "NAGALAND"
    ODISSA = "ODISSA"
    PUNJAB = "PUNJAB"
    PONDICHERRY = "PONDICHERRY"
    RAJASTHAN = "RAJASTHAN"
    SIKKIM = "SIKKIM"
    TELANGANA = "TELANGANA"
    TAMIL_NADU = "TAMIL_NADU"
    TRIPURA = "TRIPURA"
    UTTARAKHAND = "UTTARAKHAND"
    UTTAR_PRADESH = "UTTAR_PRADESH"
    WEST_BENGAL = "WEST_BENGAL"


class InquiryPurpose(Enum):
    OTHER = "OTHER"
    AUTO_LOAN = "AUTO_LOAN"
    HOUSING_LOAN = "HOUSING_LOAN"
    PROPERTY_LOAN = "PROPERTY_LOAN"
    LOAN_AGAINST_SHARES_SECURITIES = "LOAN_AGAINST_SHARES_SECURITIES"
    PERSONAL_LOAN = "PERSONAL_LOAN"
    CONSUMER_LOAN = "CONSUMER_LOAN"
    GOLD_LOAN = "GOLD_LOAN"
    EDUCATION_LOAN = "EDUCATION_LOAN"
    LOAN_TO_PROFESSIONAL = "LOAN_TO_PROFESSIONAL"
    CREDIT_CARD = "CREDIT_CARD"
    LEASE = "LEASE"
    OVERDRAFT = "OVERDRAFT"
    TWO_WHEELER_LOAN = "TWO_WHEELER_LOAN"
    NON_FUNDED_CREDIT_FACILITY = "NON_FUNDED_CREDIT_FACILITY"
    LOAN_AGAINST_BANK_DEPOSITS = "LOAN_AGAINST_BANK_DEPOSITS"
    FLEET_CARD = "FLEET_CARD"
    COMMERCIAL_VEHICLE_LOAN = "COMMERCIAL_VEHICLE_LOAN"
    TELCO_WIRELESS = "TELCO_WIRELESS"
    TELCO_BROADBAND = "TELCO_BROADBAND"
    TELCO_LANDLINE = "TELCO_LANDLINE"
    SECURED_CREDIT_CARD = "SECURED_CREDIT_CARD"
    USED_CAR_LOAN = "USED_CAR_LOAN"
    CONSTRUCTION_EQUIPMENT_LOAN = "CONSTRUCTION_EQUIPMENT_LOAN"
    TRACTOR_LOAN = "TRACTOR_LOAN"
    CORPORATE_CREDIT_CARD = "CORPORATE_CREDIT_CARD"
    AUTO_LEAS = "AUTO_LEAS"
    BUSINESS_LOAN = "BUSINESS_LOAN"
    BUSINESS_LOAN_PRIORITY_SECTOR_SMALL_BUSINESS = (
        "BUSINESS_LOAN_PRIORITY_SECTOR_SMALL_BUSINESS"
    )
    BUSINESS_LOAN_PRIORITY_SECTOR_AGRICULTURE = (
        "BUSINESS_LOAN_PRIORITY_SECTOR_AGRICULTURE"
    )
    BUSINESS_LOAN_PRIORITY_SECTOR_OTHERS = "BUSINESS_LOAN_PRIORITY_SECTOR_OTHERS"
    BUSINESS_NON_FUNDED_CREDIT_FACILITY = "BUSINESS_NON_FUNDED_CREDIT_FACILITY"
    BUSINESS_NON_FUNDED_CREDIT_FACILITY_PRIORITY_SECTOR_SMALL_BUSINESS = (
        "BUSINESS_NON_FUNDED_CREDIT_FACILITY_PRIORITY_SECTOR_SMALL_BUSINESS"
    )
    BUSINESS_NON_FUNDED_CREDIT_FACILITY_PRIORITY_SECTOR_AGRICULTURE = (
        "BUSINESS_NON_FUNDED_CREDIT_FACILITY_PRIORITY_SECTOR_AGRICULTURE"
    )
    BUSINESS_NON_FUNDED_CREDIT_FACILITY_PRIORITY_SECTOR_OTHE = (
        "BUSINESS_NON_FUNDED_CREDIT_FACILITY_PRIORITY_SECTOR_OTHE"
    )
    BUSINESS_LOAN_AGAINST_BANK_DEPOSITS = "BUSINESS_LOAN_AGAINST_BANK_DEPOSITS"
    STAFF_LOAN = "STAFF_LOAN"
    DISCLOSURE = "DISCLOSURE"
    MICROFINANCE_BUSINESS_LOAN = "MICROFINANCE_BUSINESS_LOAN"
    MICROFINANCE_PERSONAL_LOAN = "MICROFINANCE_PERSONAL_LOAN"
    MICROFINANCE_HOUSING_LOAN = "MICROFINANCE_HOUSING_LOAN"
    MICROFINANCE_OTHERS = "MICROFINANCE_OTHERS"


class IDTypes(Enum):
    AADHAR = "AADHAR"
    PAN = "PAN"
    VOTER_ID = "VOTER_ID"
    PASSPORT_ID = "PASSPORT_ID"
    RATION_CARD = "RATION_CARD"
    DRIVER_LICENSE = "DRIVER_LICENSE"
    OTHER_ID = "OTHER_ID"


class AddressType(Enum):
    HOME = "HOME"
    OFFICE = "OFFICE"
    OTHER = "OTHER"


class AdditionalNameType(Enum):
    FATHER = "FATHER"
    HUSBAND = "HUSBAND"
    BROTHER = "BROTHER"
    SON = "SON"
    SON_IN_LAW = "SON_IN_LAW"
    FATHER_IN_LAW = "FATHER_IN_LAW"
    BROTHER_IN_LAW = "BROTHER_IN_LAW"
    OTHER = "OTHER"
    MOTHER = "MOTHER"
    WIFE = "WIFE"
    SISTER = "SISTER"
    DAUGHTER = "DAUGHTER"
    DAUGHTER_IN_LAW = "DAUGHTER_IN_LAW"
    MOTHER_IN_LAW = "MOTHER_IN_LAW"
    SISTER_IN_LAW = "SISTER_IN_LAW"


class ProductCode(Enum):
    CCR = "CCR"
    MCR = "MCR"
    MCS = "MCS"



from unittest import mock
from unittest import TestCase

from mercuryclient.api import MercuryApi
from mercuryclient.types.bbps.request import (
    AgentRequest,
    BillPaymentRequest,
    BillPaymentRequestTransid,
    ComplaintStatusRequest,
    GetAmountRequest,
    ServiceComplaintRequest,
    TxnComplaintRequest,
)


class BbpsTest(TestCase):
    def setUp(self):
        self.client = MercuryApi(
            {
                "username": "username",
                "password": "password",
                "url": "https://mercury-dev.esthenos.in",
            }
        )
        self.post_api_mock = mock.patch(
            "mercuryclient.api.MercuryApi._post_json_http_request"
        ).start()
        self.addCleanup(self.post_api_mock.stop)

    def test_request_agent_onboard(self):

        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = AgentRequest(
            name="random",
            contactperson="random",
            mobileNumber="9999999999",
            agentshopname="random",
            businesstype="KIRANA_SHOP",
            address1="random address",
            address2="random address",
            state="random",
            city="random",
            pincode="123432",
            latitude="random",
            longitude="random",
            email="some@ramdom.com",
        )
        request_dict = {
            "name": "random",
            "contactperson": "random",
            "mobileNumber": "9999999999",
            "agentshopname": "random",
            "businesstype": "KIRANA_SHOP",
            "address1": "random address",
            "address2": "random address",
            "state": "random",
            "city": "random",
            "pincode": "123432",
            "latitude": "random",
            "longitude": "random",
            "email": "some@ramdom.com",
            "profile": "some_profile",
        }
        self.client.set_agent_on_board(request_obj, "some_profile")

        self.post_api_mock.assert_called_with(
            path="api/v1/bbps/agent/",
            data=request_dict,
            send_request_id=True,
            add_bearer_token=True,
        )

    def test_agent_request_exception_raised_if_status_code_error(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 401
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = AgentRequest(
            name="random",
            contactperson="random",
            mobileNumber="9999999999",
            agentshopname="random",
            businesstype="KIRANA_SHOP",
            address1="random address",
            address2="random address",
            state="random",
            city="random",
            pincode="123432",
            latitude="random",
            longitude="random",
            email="some@ramdom.com",
        )
        with self.assertRaises(Exception):
            self.client.set_agent_on_board(request_obj, "some_profile")

    def test_agent_request_api_succeeds_if_status_code_success(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = AgentRequest(
            name="random",
            contactperson="random",
            mobileNumber="9999999999",
            agentshopname="random",
            businesstype="KIRANA_SHOP",
            address1="random address",
            address2="random address",
            state="random",
            city="random",
            pincode="123432",
            latitude="random",
            longitude="random",
            email="some@ramdom.com",
        )
        request_id = self.client.set_agent_on_board(request_obj, "some_profile")
        self.assertEqual(request_id["request_id"], "random_string")

    def test_bill_categories_request(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = {"coverage": "IND"}
        request_dict = {"coverage": "IND", "profile": "some_profile"}
        self.client.get_bill_categories(request_obj, "some_profile")

        self.post_api_mock.assert_called_with(
            path="api/v1/bbps/bill_categories/",
            data=request_dict,
            send_request_id=True,
            add_bearer_token=True,
        )

    def test_bill_categories_request_exception_raised_if_status_code_error(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 401
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = {"coverage": "IND"}
        with self.assertRaises(Exception):
            self.client.get_bill_categories(request_obj, "some_profile")

    def test_bill_categories_request_api_succeeds_if_status_code_success(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = {"coverage": "IND"}
        request_id = self.client.get_bill_categories(request_obj, "some_profile")
        self.assertEqual(request_id["request_id"], "random_string")

    def test_biller_by_categories_request(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = {"categoryId": "3"}
        request_dict = {"categoryId": "3", "profile": "some_profile"}
        self.client.get_biller_by_categories(request_obj, "some_profile")

        self.post_api_mock.assert_called_with(
            path="api/v1/bbps/biller/",
            data=request_dict,
            send_request_id=True,
            add_bearer_token=True,
        )

    def test_biller_by_categories_request_exception_raised_if_status_code_error(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 401
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = {"categoryId": "3"}
        with self.assertRaises(Exception):
            self.client.get_biller_by_categories(request_obj, "some_profile")

    def test_biller_by_categories_request_api_succeeds_if_status_code_success(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = {"categoryId": "3"}
        request_id = self.client.get_biller_by_categories(request_obj, "some_profile")
        self.assertEqual(request_id["request_id"], "random_string")

    def test_customer_params_by_biller_request(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = {"billerId": "MAHA00000MAH01"}
        request_dict = {"billerId": "MAHA00000MAH01", "profile": "some_profile"}
        self.client.get_customer_params_by_biller_id(request_obj, "some_profile")

        self.post_api_mock.assert_called_with(
            path="api/v1/bbps/customer_by_biller/",
            data=request_dict,
            send_request_id=True,
            add_bearer_token=True,
        )

    def test_customer_params_by_biller_request_exception_raised_if_status_code_error(
        self,
    ):
        mock_response = mock.MagicMock()
        mock_response.status_code = 401
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = {"billerId": "MAHA00000MAH01"}
        with self.assertRaises(Exception):
            self.client.get_customer_params_by_biller_id(request_obj, "some_profile")

    def test_customer_params_by_biller_request_api_succeeds_if_status_code_success(
        self,
    ):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = {"billerId": "MAHA00000MAH01"}
        request_id = self.client.get_customer_params_by_biller_id(
            request_obj, "some_profile"
        )
        self.assertEqual(request_id["request_id"], "random_string")

    def test_get_amount_request(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = GetAmountRequest(
            billerId="random",
            mobileNumber="9999999999",
            crno="random",
            agentId="random",
        )
        request_dict = {
            "billerId": "random",
            "mobileNumber": "9999999999",
            "crno": "random",
            "agentId": "random",
            "profile": "some_profile",
        }
        self.client.get_amount(request_obj, "some_profile")

        self.post_api_mock.assert_called_with(
            path="api/v1/bbps/amount/",
            data=request_dict,
            send_request_id=True,
            add_bearer_token=True,
        )

    def test_get_amount_request_exception_raised_if_status_code_error(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 401
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = GetAmountRequest(
            billerId="random",
            mobileNumber="9999999999",
            crno="random",
            agentId="random",
        )
        with self.assertRaises(Exception):
            self.client.get_amount(request_obj, "some_profile")

    def test_get_amount_request_api_succeeds_if_status_code_success(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = GetAmountRequest(
            billerId="random",
            mobileNumber="9999999999",
            crno="random",
            agentId="random",
        )
        request_id = self.client.get_amount(request_obj, "some_profile")
        self.assertEqual(request_id["request_id"], "random_string")

    def test_bill_payment_request_transid_request(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = BillPaymentRequestTransid(
            billAmount="random",
            transid="random",
            mobileNumber="9999999999",
            agentId="random",
            BillerCategory="random",
            billerId="random",
            crno="random",
        )
        request_dict = {
            "billAmount": "random",
            "transid": "random",
            "mobileNumber": "9999999999",
            "agentId": "random",
            "BillerCategory": "random",
            "billerId": "random",
            "crno": "random",
            "profile": "some_profile",
        }
        self.client.send_bill_payment_request_transid(request_obj, "some_profile")

        self.post_api_mock.assert_called_with(
            path="api/v1/bbps/payment_transid/",
            data=request_dict,
            send_request_id=True,
            add_bearer_token=True,
        )

    def test_bill_payment_request_transid_request_exception_raised_if_status_code_error(
        self,
    ):
        mock_response = mock.MagicMock()
        mock_response.status_code = 401
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = BillPaymentRequestTransid(
            billAmount="random",
            transid="random",
            mobileNumber="9999999999",
            agentId="random",
            BillerCategory="random",
            billerId="random",
            crno="random",
        )
        with self.assertRaises(Exception):
            self.client.send_bill_payment_request_transid(request_obj, "some_profile")

    def test_bill_payment_request_transid_request_api_succeeds_if_status_code_success(
        self,
    ):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = BillPaymentRequestTransid(
            billAmount="random",
            transid="random",
            mobileNumber="9999999999",
            agentId="random",
            BillerCategory="random",
            billerId="random",
            crno="random",
        )
        request_id = self.client.send_bill_payment_request_transid(
            request_obj, "some_profile"
        )
        self.assertEqual(request_id["request_id"], "random_string")

    def test_bill_payment_request(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = BillPaymentRequest(
            billAmount="random",
            mobileNumber="9999999999",
            agentId="random",
            BillerCategory="random",
            billerId="random",
            crno="random",
        )
        request_dict = {
            "billAmount": "random",
            "mobileNumber": "9999999999",
            "agentId": "random",
            "BillerCategory": "random",
            "billerId": "random",
            "crno": "random",
            "profile": "some_profile",
        }
        self.client.send_bill_payment_request(request_obj, "some_profile")

        self.post_api_mock.assert_called_with(
            path="api/v1/bbps/payment/",
            data=request_dict,
            send_request_id=True,
            add_bearer_token=True,
        )

    def test_bill_payment_request_exception_raised_if_status_code_error(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 401
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = BillPaymentRequest(
            billAmount="random",
            mobileNumber="9999999999",
            agentId="random",
            BillerCategory="random",
            billerId="random",
            crno="random",
        )
        with self.assertRaises(Exception):
            self.client.send_bill_payment_request(request_obj, "some_profile")

    def test_bill_payment_request_request_api_succeeds_if_status_code_success(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = BillPaymentRequest(
            billAmount="random",
            mobileNumber="9999999999",
            agentId="random",
            BillerCategory="random",
            billerId="random",
            crno="random",
        )
        request_id = self.client.send_bill_payment_request(request_obj, "some_profile")
        self.assertEqual(request_id["request_id"], "random_string")

    def test_duplicate_payment_receipt_request(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = {"refId": "random"}
        request_dict = {"refId": "random", "profile": "some_profile"}
        self.client.get_duplicate_payment_receipt(request_obj, "some_profile")

        self.post_api_mock.assert_called_with(
            path="api/v1/bbps/payment_receipt/",
            data=request_dict,
            send_request_id=True,
            add_bearer_token=True,
        )

    def test_duplicate_payment_receipt_exception_raised_if_status_code_error(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 401
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = {"refId": "random"}
        with self.assertRaises(Exception):
            self.client.get_duplicate_payment_receipt(request_obj, "some_profile")

    def test_duplicate_payment_receipt_request_api_succeeds_if_status_code_success(
        self,
    ):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = {"refId": "random"}
        request_id = self.client.get_duplicate_payment_receipt(
            request_obj, "some_profile"
        )
        self.assertEqual(request_id["request_id"], "random_string")

    def test_trasaction_complaint_request(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = TxnComplaintRequest(
            transactionRefId="random",
            mobileNumber="1111111111",
            reason="random",
            description="random",
        )
        request_dict = {
            "transactionRefId": "random",
            "mobileNumber": "1111111111",
            "reason": "random",
            "description": "random",
            "profile": "some_profile",
        }
        self.client.register_trasaction_complaint(request_obj, "some_profile")

        self.post_api_mock.assert_called_with(
            path="api/v1/bbps/trasaction_complaint/",
            data=request_dict,
            send_request_id=True,
            add_bearer_token=True,
        )

    def test_trasaction_complaint_exception_raised_if_status_code_error(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 401
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = TxnComplaintRequest(
            transactionRefId="random",
            mobileNumber="1111111111",
            reason="random",
            description="random",
        )
        with self.assertRaises(Exception):
            self.client.register_trasaction_complaint(request_obj, "some_profile")

    def test_trasaction_complaint_request_api_succeeds_if_status_code_success(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = TxnComplaintRequest(
            transactionRefId="random",
            mobileNumber="1111111111",
            reason="random",
            description="random",
        )
        request_id = self.client.register_trasaction_complaint(
            request_obj, "some_profile"
        )
        self.assertEqual(request_id["request_id"], "random_string")

    def test_service_complaint_request(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = ServiceComplaintRequest(
            billerId="random",
            mobileNumber="1111111111",
            description="random",
            reason="random",
            type="random",
        )
        request_dict = {
            "billerId": "random",
            "mobileNumber": "1111111111",
            "description": "random",
            "reason": "random",
            "type": "random",
            "profile": "some_profile",
        }
        self.client.register_service_complaint(request_obj, "some_profile")

        self.post_api_mock.assert_called_with(
            path="api/v1/bbps/service_complaint/",
            data=request_dict,
            send_request_id=True,
            add_bearer_token=True,
        )

    def test_service_complaint_exception_raised_if_status_code_error(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 401
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = ServiceComplaintRequest(
            billerId="random",
            mobileNumber="1111111111",
            description="random",
            reason="random",
            type="random",
        )
        with self.assertRaises(Exception):
            self.client.register_service_complaint(request_obj, "some_profile")

    def test_service_complaint_request_api_succeeds_if_status_code_success(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = ServiceComplaintRequest(
            billerId="random",
            mobileNumber="1111111111",
            description="random",
            reason="random",
            type="random",
        )
        request_id = self.client.register_service_complaint(request_obj, "some_profile")
        self.assertEqual(request_id["request_id"], "random_string")

    def test_send_complaint_status_request(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = ComplaintStatusRequest(
            complaintType="Service", complaintId="random"
        )
        request_dict = {
            "complaintType": "Service",
            "complaintId": "random",
            "profile": "some_profile",
        }
        self.client.get_complaint_status(request_obj, "some_profile")

        self.post_api_mock.assert_called_with(
            path="api/v1/bbps/complaint_status/",
            data=request_dict,
            send_request_id=True,
            add_bearer_token=True,
        )

    def test_complaint_status_exception_raised_if_status_code_error(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 401
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = ComplaintStatusRequest(
            complaintType="Service", complaintId="random"
        )
        with self.assertRaises(Exception):
            self.client.get_complaint_status(request_obj, "some_profile")

    def test_complaint_status_request_api_succeeds_if_status_code_success(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_obj = ComplaintStatusRequest(
            complaintType="Service", complaintId="random"
        )
        request_id = self.client.get_complaint_status(request_obj, "some_profile")
        self.assertEqual(request_id["request_id"], "random_string")

    def test_get_bbpsid_request(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_dict = {"profile": "some_profile"}
        self.client.get_bbpsid("some_profile")

        self.post_api_mock.assert_called_with(
            path="api/v1/bbps/bbpsid/",
            data=request_dict,
            send_request_id=True,
            add_bearer_token=True,
        )

    def test_get_bbpsid_exception_raised_if_status_code_error(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 401
        self.post_api_mock.return_value = ("random_string", mock_response)
        with self.assertRaises(Exception):
            self.client.get_bbpsid("some_profile")

    def test_get_bbpsid_request_api_succeeds_if_status_code_success(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_id = self.client.get_bbpsid("some_profile")
        self.assertEqual(request_id["request_id"], "random_string")



from enum import Enum


class ShopType(Enum):

    KIRANA_SHOP = "KIRANA_SHOP"
    MOBILE_SHOP = "MOBILE_SHOP"
    COPIER_SHOP = "COPIER_SHOP"
    INTERNET_CAFE = "INTERNET_CAFE"
    OTHER = "OTHER"


class ComplaintType(Enum):
    Transaction = "Transaction"
    Service = "Service"



import time


class UdyogAadhaarVerifyMixin:
    """
    Mixin for Udyog Aadhaar verification
    """

    def initiate_udyog_verification(self, request_dict):
        """
        udyog verification using the mercury service

        :param udyog_aadhaar: string
        :param provider: any
        :param profile: An existing profile in Mercury. The profile has to correspond
        to the provider.
        :return: (request_id, data)
        """
        api = "api/v1/udyog_aadhaar/verify/"

        request_id, r = self._post_json_http_request(
            api, data=request_dict, send_request_id=True, add_bearer_token=True
        )
        try:
            response_json = r.json()
        except Exception:
            response_json = {}

        if r.status_code == 201:
            return request_id

        raise Exception(
            f"Error while sending Udyog aadhaar verify request. Status: {r.status_code}, Response is {response_json}"
        )

    def get_udyog_details(self, request_id: str):
        api = "api/v1/udyog_aadhaar/verify/"

        request_id, r = self._get_json_http_request(
            api,
            headers={"X-Mercury-Request-Id": request_id},
            send_request_id=False,
            add_bearer_token=True,
        )

        if r.status_code == 200:
            result = r.json()
            if result["status"] == "FAILURE":
                raise Exception(
                    f"Error verifying Udyog aadhaar. Status: {result['status']} | Message {result['message']}"
                )
            return request_id, result

        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            f"Error getting Udyog aadhaar Verification result. Status: {r.status_code}, Response is {response_json}"
        )

    def verify_udyog_aadhaar(
        self,
        udyog_aadhaar: str,
        consent: bool,
        provider: str,
        profile: str,
        max_attempts: int = 3,
        retry_backoff: int = 5,
    ):
        """Based on udyog aadhaar numbers it fetches all the details associated with it
        returns: mercury_request_id, verified udyog response from vendor
        """
        request_data = {
            "provider": provider,
            "profile": profile,
            "udyog_aadhaar": udyog_aadhaar,
            "consent": consent,
        }

        mercury_request_id = self.initiate_udyog_verification(request_data)

        for attempts in range(max_attempts):
            time.sleep(retry_backoff)
            mercury_request_id, result = self.get_udyog_details(mercury_request_id)
            if result.get("status") != "IN_PROGRESS":
                return mercury_request_id, result

            retry_backoff *= 2

        raise Exception("Error while getting udyog response. Status: IN_PROGRESS")



from datetime import date
from unittest import TestCase, mock

from mercuryclient.api import MercuryApi
from mercuryclient.types.equifax.request import EquifaxRequest


class EquifaxMixinTest(TestCase):
    def setUp(self):
        self.client = MercuryApi(
            {
                "username": "username",
                "password": "password",
                "url": "https://mercury-dev.esthenos.in",
            }
        )

        self.post_api_mock = mock.patch(
            "mercuryclient.api.MercuryApi._post_json_http_request"
        ).start()
        self.addCleanup(self.post_api_mock.stop)
        self.get_api_mock = mock.patch(
            "mercuryclient.api.MercuryApi._get_json_http_request"
        ).start()
        self.addCleanup(self.get_api_mock.stop)
        data = {
            "profile": "test",
            "product_code": ["CCR"],
            "inquiry_purpose": "OTHER",
            "first_name": "STEVE SMITH",
            "middle_name": "",
            "last_name": "SMITH",
            "dob": date(1996, 12, 9),
            "inquiry_address": [
                {
                    "address_type": ["HOME"],
                    "address_line_1": "B/404, Durja Pooj Society, Haridwar, Haridwar, UP",
                    "state": "TRIPURA",
                    "city": "Udaipur",
                    "postal": "955567",
                }
            ],
            "inquiry_phones": [{"number": "7777794563", "number_type": ["MOBILE"]}],
            "id_details": [
                {"id_type": "AADHAR", "id_value": "986249396345"},
                {"id_type": "VOTER_ID", "id_value": "KJG0226988", "source": "Inquiry"},
            ],
            "mif_details": {
                "family_details": [
                    {"additional_name_type": "FATHER", "additional_name": "John Smith"},
                    {"additional_name_type": "HUSBAND", "additional_name": "KIng"},
                ]
            },
        }

        self.request_data = EquifaxRequest(**data)

    def test_request_equifax_request(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        self.client.request_equifax_report(self.request_data, "test")
        self.post_api_mock.assert_called_with(
            "api/v1/equifax/",
            data={
                "product_code": ["CCR"],
                "inquiry_purpose": "OTHER",
                "first_name": "STEVE SMITH",
                "middle_name": "",
                "last_name": "SMITH",
                "dob": "1996-12-09",
                "inquiry_address": [
                    {
                        "address_type": ["HOME"],
                        "address_line_1": "B/404, Durja Pooj Society, Haridwar, Haridwar, UP",
                        "state": "TRIPURA",
                        "city": "Udaipur",
                        "postal": "955567",
                    }
                ],
                "inquiry_phones": [{"number": "7777794563", "number_type": ["MOBILE"]}],
                "id_details": [
                    {"id_type": "AADHAR", "id_value": "986249396345"},
                    {
                        "id_type": "VOTER_ID",
                        "id_value": "KJG0226988",
                        "source": "Inquiry",
                    },
                ],
                "mif_details": {
                    "family_details": [
                        {
                            "additional_name_type": "FATHER",
                            "additional_name": "John Smith",
                        },
                        {"additional_name_type": "HUSBAND", "additional_name": "KIng"},
                    ]
                },
                "profile": "test",
            },
            send_request_id=True,
            add_bearer_token=True,
        )

    def test_request_exception_raised_if_status_code_error(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 401
        self.post_api_mock.return_value = ("random_string", mock_response)
        with self.assertRaises(Exception):
            self.client.request_equifax_report(self.request_data, "test")

    def test_request_api_succeeds_if_status_code_success(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)

        request_id = self.client.request_equifax_report(self.request_data, "test")
        self.assertEqual(request_id, "random_string")

    def test_response_verify_rc(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 200
        self.get_api_mock.return_value = ("random_string", mock_response)
        self.client.get_equifax_response("random_string")

        self.get_api_mock.assert_called_with(
            "api/v1/equifax/",
            headers={"X-Mercury-Request-Id": "random_string"},
            send_request_id=False,
            add_bearer_token=True,
        )

    def test_response_exception_raised_if_status_code_error(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 401
        self.get_api_mock.return_value = ("random_string", mock_response)
        with self.assertRaises(Exception):
            self.client.get_equifax_response("random_string")

    def test_response_api_succeeds_if_status_code_success(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 200
        mock_response.json = mock.MagicMock()
        mock_response.json.return_value = {"status": "SUCCESS"}
        self.get_api_mock.return_value = ("random_string", mock_response)

        request_id, result = self.client.get_equifax_response("random_string")
        self.assertEqual(request_id, "random_string")
        self.assertEqual(result["status"], "SUCCESS")



from unittest import mock
from unittest import TestCase

from mercuryclient.api import MercuryApi


class AuthMixinTest(TestCase):
    def setUp(self):
        self.post_api_mock = mock.patch(
            "mercuryclient.api.MercuryApi._post_json_http_request"
        ).start()
        self.addCleanup(self.post_api_mock.stop)

    def test_authorizing_client(self):
        client = MercuryApi(
            {
                "username": "username",
                "password": "password",
                "url": "https://mercury-dev.esthenos.in",
            }
        )
        mock_response = mock.MagicMock()
        mock_response.status_code = 200
        mock_response.json = mock.MagicMock()
        mock_response.json.return_value = {
            "refresh": "refresh_token",
            "access": "access_token",
        }
        self.post_api_mock.return_value = ("random_string", mock_response)

        client._authorize_client()

        self.post_api_mock.assert_called_with(
            "api/v1/token/",
            data={"username": "username", "password": "password"},
            send_request_id=True,
            add_bearer_token=False,
        )

        self.post_api_mock.reset_mock()
        mock_response = mock.MagicMock()
        mock_response.status_code = 403
        self.post_api_mock.return_value = ("random_string", mock_response)

        with self.assertRaises(Exception):
            client._authorize_client()

        self.post_api_mock.assert_called_with(
            "api/v1/token/",
            data={"username": "username", "password": "password"},
            send_request_id=True,
            add_bearer_token=False,
        )

    def test_refreshing_access_token(self):
        client = MercuryApi(
            {
                "username": "username",
                "password": "password",
                "url": "https://mercury-dev.esthenos.in",
            }
        )
        client._refresh_token = "refresh_token"
        mock_response = mock.MagicMock()
        mock_response.status_code = 200
        mock_response.json = mock.MagicMock()
        mock_response.json.return_value = {"access": "access_token"}
        self.post_api_mock.return_value = ("random_string", mock_response)

        client.set_access_token()

        self.post_api_mock.assert_called_with(
            "api/v1/token/refresh/",
            data={"refresh": "refresh_token"},
            send_request_id=True,
            add_bearer_token=False,
        )

        self.post_api_mock.reset_mock()
        mock_response = mock.MagicMock()
        mock_response.status_code = 403
        self.post_api_mock.return_value = ("random_string", mock_response)

        with self.assertRaises(Exception):
            client.set_access_token()

        self.post_api_mock.assert_called_with(
            "api/v1/token/refresh/",
            data={"refresh": "refresh_token"},
            send_request_id=True,
            add_bearer_token=False,
        )



import json
from mercuryclient.types.bbps.request import (
    AgentRequest,
    BillPaymentRequest,
    BillPaymentRequestTransid,
    ComplaintStatusRequest,
    GetAmountRequest,
    ServiceComplaintRequest,
    TxnComplaintRequest,
)


class BbpsMixin:
    def set_agent_on_board(self, request_obj: AgentRequest, profile: str):
        """
        Create Agent using the mercury service

        :param: request_obj: AgentRequest
        :param profile: An existing profile in Mercury.
        :return: (request_id, status,data )
        """

        api = "api/v1/bbps/agent/"

        request_dict = json.loads(request_obj.json(exclude_unset=True))
        request_dict["profile"] = profile

        request_id, r = self._post_json_http_request(
            path=api, data=request_dict, send_request_id=True, add_bearer_token=True
        )

        if r.status_code == 201:

            return {"request_id": request_id, "data": r.content}

        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error while sending Agent Request Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )

    def get_state(self, profile: str):
        """
        Ftech state using the mercury service

        :param profile: An existing profile in Mercury.
        :return: (request_id, status,data )
        """

        api = "api/v1/bbps/state/"

        request_dict = {}
        request_dict["profile"] = profile

        request_id, r = self._post_json_http_request(
            path=api, data=request_dict, send_request_id=True, add_bearer_token=True
        )

        if r.status_code == 201:

            return {"request_id": request_id, "data": r.content}

        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error while sending Status Request Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )

    def get_district_by_state(self, request_obj: dict, profile: str):
        """
        Fetch Status  using the mercury service

        :param: request_obj: dict
        :param profile: An existing profile in Mercu
        ry.
        :return: (request_id, status,data )
        """

        api = "api/v1/bbps/distrct/"

        request_dict = request_obj
        request_dict["profile"] = profile

        request_id, r = self._post_json_http_request(
            path=api, data=request_dict, send_request_id=True, add_bearer_token=True
        )

        if r.status_code == 201:

            return {"request_id": request_id, "data": r.content}

        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error while sending District Request Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )

    def get_bill_categories(self, request_obj: dict, profile: str):
        """
        Ftech Bill Categories  using the mercury service

        :param: request_obj: dict
        :param profile: An existing profile in Mercury.
        :return: (request_id, status,data )
        """

        api = "api/v1/bbps/bill_categories/"

        request_dict = request_obj
        request_dict["profile"] = profile

        request_id, r = self._post_json_http_request(
            path=api, data=request_dict, send_request_id=True, add_bearer_token=True
        )

        if r.status_code == 201:

            return {"request_id": request_id, "data": r.content}

        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error while sending bill categories Request Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )

    def get_biller_by_categories(self, request_obj: dict, profile: str):
        """
        Fetch billers using the mercury service

        :param: request_obj: dict
        :param profile: An existing profile in Mercury.
        :return: (request_id, status,data )
        """

        api = "api/v1/bbps/biller/"

        request_dict = request_obj
        request_dict["profile"] = profile

        request_id, r = self._post_json_http_request(
            path=api, data=request_dict, send_request_id=True, add_bearer_token=True
        )

        if r.status_code == 201:

            return {"request_id": request_id, "data": r.content}

        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error while sending Biller Request Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )

    def get_customer_params_by_biller_id(self, request_obj: dict, profile: str):
        """
        Fetch customer params using the mercury service

        :param: request_obj: dict
        :param profile: An existing profile in Mercury.
        :return: (request_id, status,data )
        """

        api = "api/v1/bbps/customer_by_biller/"

        request_dict = request_obj
        request_dict["profile"] = profile

        request_id, r = self._post_json_http_request(
            path=api, data=request_dict, send_request_id=True, add_bearer_token=True
        )

        if r.status_code == 201:

            return {"request_id": request_id, "data": r.content}

        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error while sending customer params Request Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )

    def get_amount(self, request_obj: GetAmountRequest, profile: str):
        """
        Fetching Amount  using the mercury service

        :param: request_obj: GetAmountRequest
        :param profile: An existing profile in Mercury.
        :return: (request_id, status,data )
        """

        api = "api/v1/bbps/amount/"

        request_dict = json.loads(request_obj.json(exclude_unset=True))
        request_dict["profile"] = profile

        request_id, r = self._post_json_http_request(
            path=api, data=request_dict, send_request_id=True, add_bearer_token=True
        )

        if r.status_code == 201:

            return {"request_id": request_id, "data": r.content}

        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error while sending get amount Request Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )

    def send_bill_payment_request_transid(
        self, request_obj: BillPaymentRequestTransid, profile: str
    ):
        """
        Bill payment with trans id using the mercury service

        :param: request_obj: BillPaymentRequestTransid
        :param profile: An existing profile in Mercury.
        :return: (request_id, status,data )
        """

        api = "api/v1/bbps/payment_transid/"

        request_dict = json.loads(request_obj.json(exclude_unset=True))
        request_dict["profile"] = profile

        request_id, r = self._post_json_http_request(
            path=api, data=request_dict, send_request_id=True, add_bearer_token=True
        )

        if r.status_code == 201:

            return {"request_id": request_id, "data": r.content}

        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error while sending Bbps Request Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )

    def send_bill_payment_request(self, request_obj: BillPaymentRequest, profile: str):
        """
        Send bill payment using the mercury service

        :param: request_obj: BillPaymentRequest
        :param profile: An existing profile in Mercury.
        :return: (request_id, status,data )
        """

        api = "api/v1/bbps/payment/"

        request_dict = json.loads(request_obj.json(exclude_unset=True))
        request_dict["profile"] = profile

        request_id, r = self._post_json_http_request(
            path=api, data=request_dict, send_request_id=True, add_bearer_token=True
        )

        if r.status_code == 201:

            return {"request_id": request_id, "data": r.content}

        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error while sending bill payment Request Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )

    def get_duplicate_payment_receipt(self, request_obj: dict, profile: str):
        """
        Fetch duplicate receipt using the mercury service

        :param: request_obj: Dict
        :param profile: An existing profile in Mercury.
        :return: (request_id, status,data )
        """

        api = "api/v1/bbps/payment_receipt/"

        request_dict = request_obj
        request_dict["profile"] = profile

        request_id, r = self._post_json_http_request(
            path=api, data=request_dict, send_request_id=True, add_bearer_token=True
        )

        if r.status_code == 201:

            return {"request_id": request_id, "data": r.content}

        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error while fetching duplicate receipt Request Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )

    def register_trasaction_complaint(
        self, request_obj: TxnComplaintRequest, profile: str
    ):
        """
        Register trasaction complaint using the mercury service

        :param: request_obj: TxnComplaintRequest
        :param profile: An existing profile in Mercury.
        :return: (request_id, status,data )
        """

        api = "api/v1/bbps/trasaction_complaint/"

        request_dict = json.loads(request_obj.json(exclude_unset=True))
        request_dict["profile"] = profile

        request_id, r = self._post_json_http_request(
            path=api, data=request_dict, send_request_id=True, add_bearer_token=True
        )

        if r.status_code == 201:

            return {"request_id": request_id, "data": r.content}

        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error while sending trasaction complaint Request Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )

    def register_service_complaint(
        self, request_obj: ServiceComplaintRequest, profile: str
    ):
        """
        Service complaint using the mercury service

        :param: request_obj: ServiceComplaintRequest
        :param profile: An existing profile in Mercury.
        :return: (request_id, status,data )
        """

        api = "api/v1/bbps/service_complaint/"

        request_dict = json.loads(request_obj.json(exclude_unset=True))
        request_dict["profile"] = profile

        request_id, r = self._post_json_http_request(
            path=api, data=request_dict, send_request_id=True, add_bearer_token=True
        )

        if r.status_code == 201:

            return {"request_id": request_id, "data": r.content}

        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error while sending Service complaint Request Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )

    def get_complaint_status(self, request_obj: ComplaintStatusRequest, profile: str):
        """
        Fetch complaint status using the mercury service

        :param: request_obj: ComplaintStatusRequest
        :param profile: An existing profile in Mercury.
        :return: (request_id, status,data )
        """

        api = "api/v1/bbps/complaint_status/"

        request_dict = json.loads(request_obj.json(exclude_unset=True))
        request_dict["profile"] = profile

        request_id, r = self._post_json_http_request(
            path=api, data=request_dict, send_request_id=True, add_bearer_token=True
        )

        if r.status_code == 201:

            return {"request_id": request_id, "data": r.content}

        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error while sending Complaint Status Request Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )

    def get_bbpsid(self, profile: str):
        """
        Fetch bbps id  using the mercury service

        :param profile: An existing profile in Mercury.
        :return: (request_id, status,data )
        """

        api = "api/v1/bbps/bbpsid/"

        request_dict = {}
        request_dict["profile"] = profile

        request_id, r = self._post_json_http_request(
            path=api, data=request_dict, send_request_id=True, add_bearer_token=True
        )

        if r.status_code == 201:

            return {"request_id": request_id, "data": r.content}

        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error while sending bbps id Request Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )



class SMSMixin:
    """
    Mixin for sending emails
    """

    def send_sms(self, recipient, message, provider, profile):
        """
        POST a request to Mercury to send an SMS

        :param recipient: Phone number of recipients
        :param message: SMS message to be sent
        :param provider: Name of SMS Provider
        :param profile: Profile name from which to send SMS
        :return: (request_id, status)
        """
        api = "api/v1/sms/"

        data = {
            "provider": provider,
            "profile": profile,
            "recipient": recipient,
            "message": message,
        }

        request_id, r = self._post_json_http_request(
            api, data=data, send_request_id=True, add_bearer_token=True
        )

        if r.status_code == 201:
            return {"request_id": request_id, "status": "Success"}

        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error while sending SMS. Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )

    def get_sms_result(self, request_id):
        """
        Get result for SMS request at the provided request ID. You can poll this method
        to check for response. If the job is still progressing, the status within the
        response will be IN_PROGRESS and the method can continue to be polled until you
        get either a SUCCESS or FAILURE

        :param request_id: Request ID of the SMS request
        :type request_id: str
        :return: Dict containing request ID, status and job response within "response".
            This response will contain the "status" of the job (IN_PROGRESS, SUCCESS or
            FAILURE), a "message" and a "data" key containing a dict with the bureau
            report
        :rtype: dict
        """
        api = "api/v1/sms/"

        request_id, r = self._get_json_http_request(
            api,
            headers={"X-Mercury-Request-Id": request_id},
            send_request_id=False,
            add_bearer_token=True,
        )

        if r.status_code == 200:
            result = r.json()
            if result["status"] == "FAILURE":
                raise Exception(
                    "Error sending SMS. Status: {} | Message {}".format(
                        result["status"], result["message"]
                    )
                )
            return {
                "request_id": request_id,
                "status": result["status"],
                "data": result,
            }

        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error getting SMS result. Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )



from datetime import date
from typing import List
from typing import Optional

from pydantic import BaseModel
from pydantic import Field
from pydantic import root_validator

from ..common import NormalizedString
from ..common import Pincode
from .enums import AddressType
from .enums import Gender
from .enums import IdentityType
from .enums import PhoneNumberType
from .enums import Purpose
from .enums import RelationType
from .enums import ResidenceType
from .enums import ScoreType
from .enums import State


class Identity(BaseModel):
    id_number: NormalizedString
    id_type: IdentityType


class PhoneNumber(BaseModel):
    phone_number: NormalizedString
    number_type: PhoneNumberType


class Address(BaseModel):
    address_type: Optional[AddressType]
    address_line_1: NormalizedString
    address_line_2: Optional[NormalizedString]
    city: NormalizedString
    pincode: Pincode
    residence_type: Optional[ResidenceType]
    state: State


class Relation(BaseModel):
    relation_type: RelationType
    relation_name: NormalizedString


class Applicant(BaseModel):
    first_name: NormalizedString
    middle_name: Optional[NormalizedString]
    last_name: NormalizedString
    date_of_birth: date
    gender: Gender
    email: Optional[NormalizedString]
    company_name: Optional[NormalizedString]
    identities: Optional[List[Identity]]
    other_identities: Optional[List[Identity]] = Field(max_items=3)
    phone_numbers: Optional[List[PhoneNumber]]
    addresses: List[Address] = Field(min_items=1)
    nominee: Optional[Relation]
    key_person: Optional[Relation]
    other_relations: Optional[List[Relation]]
    account_numbers: Optional[List[NormalizedString]]

    @root_validator
    def check_one_relation(cls, values: dict):
        identities = list()
        identities.extend(values.get("identities", []) or [])
        identities.extend(values.get("other_identities", []) or [])
        identities.extend(values.get("phone_numbers", []) or [])
        assert any(
            identities
        ), "at least one of the following is required: identities, other_identities, phone_numbers"

        return values


class CibilRequest(BaseModel):
    purpose: Purpose
    amount: int
    gst_state: State
    score_type: ScoreType
    applicant: Applicant



import uuid
from urllib.parse import urljoin

from .http_utils import get_session
from .mixins import (
    AuthMixin,
    BankStatementMixin,
    CibilMixin,
    ExperianMixin,
    HighmarkMixin,
    IDVerificationMixin,
    MailMixin,
    SMSMixin,
    WebhookMixin,
    InsuranceMixin,
    RechargeMixin,
    BbpsMixin,
    AccountVerifyMixin,
    GstVerifyMixin,
    UdyogAadhaarVerifyMixin,
    OkycVerificationMixin,
    RcVerificationMixin,
    EquifaxMixin,
    RekognitionMixins,
    ItrVerificationMixin,
    ItrExtractionMixin,
    EpfoVerificationMixin,
    NameMatchMixin,
)


class MercuryApi(
    AuthMixin,
    BankStatementMixin,
    CibilMixin,
    ExperianMixin,
    HighmarkMixin,
    IDVerificationMixin,
    MailMixin,
    SMSMixin,
    WebhookMixin,
    InsuranceMixin,
    RechargeMixin,
    BbpsMixin,
    AccountVerifyMixin,
    GstVerifyMixin,
    UdyogAadhaarVerifyMixin,
    OkycVerificationMixin,
    RcVerificationMixin,
    EquifaxMixin,
    RekognitionMixins,
    ItrVerificationMixin,
    ItrExtractionMixin,
    EpfoVerificationMixin,
    NameMatchMixin,
):
    """
    Mercury client to connect to the mercury service

    """

    def __init__(self, conn_params=None):
        self.username = conn_params["username"]
        self.password = conn_params["password"]
        self.host = conn_params["url"]
        self.verify = conn_params.get("verify", True)
        self.timeout = conn_params.get("timeout", 60)
        self._access_token = None
        self._refresh_token = None
        self._session = None

    @property
    def session(self):
        if self._session is None:
            self._session = get_session()

        return self._session

    @staticmethod
    def _generate_mercury_request_id():
        """
        Generate Unique request id
        :return:
        """
        return uuid.uuid4().hex

    def _get_request_url_from_path(self, path):
        url = urljoin(self.host, path)
        return url

    def _post_json_http_request(
        self, path, data, headers=None, send_request_id=True, add_bearer_token=True
    ):
        """
        Perform POST request on the provided API with the provided data

        :param path: The path to send the HTTP POST. This will be combined with the
            configured host
        :param data: dict to be sent as JSON in the HTTP body
        :param headers: headers for the request
        :param send_request_id: Defaults to True. Indicates if a request Id has to
        generated for the request
        :param add_bearer_token: Defaults to True. If True, we will add the bearer token
        to the request
        :return: (request_id, response): The request id (if present) and the response
        """
        if headers is None:
            headers = dict()

        if send_request_id:
            headers["X-Mercury-Request-Id"] = self._generate_mercury_request_id()

        if add_bearer_token:
            headers["Authorization"] = "Bearer {}".format(self.access_token)

        url = self._get_request_url_from_path(path)
        r = self.session.post(
            url=url,
            json=data,
            headers=headers,
            timeout=self.timeout,
            verify=self.verify,
        )

        return headers.get("X-Mercury-Request-Id"), r

    def _get_json_http_request(
        self, path, headers=None, send_request_id=True, add_bearer_token=True
    ):
        """
        Perform GET request on the provided URL

        :param url: The path to send the HTTP POST. This will be combined with the
            configured host
        :type url: str
        :param headers: Headers for the request, defaults to None
        :type headers: dict, optional
        :param send_request_id: Indicates if a request ID has to be generated for the
        request, defaults to True
        :type send_request_id: bool, optional
        :param add_bearer_token: If True, will add a bearer token to the request,
        defaults to True
        :type add_bearer_token: bool, optional
        :return: (request_id, response) The request ID and the response
        :rtype: (str, str)
        """
        if headers is None:
            headers = dict()

        if send_request_id:
            headers["X-Mercury-Request-Id"] = self._generate_mercury_request_id()

        if add_bearer_token:
            headers["Authorization"] = "Bearer {}".format(self.access_token)

        url = self._get_request_url_from_path(path)
        r = self.session.get(
            url=url, headers=headers, timeout=self.timeout, verify=self.verify
        )

        return headers.get("X-Mercury-Request-Id"), r



from datetime import date
from unittest import TestCase, mock

from mercuryclient.api import MercuryApi
from mercuryclient.types.id_verification.request import PassportDetails


class IDVerificationMixinTest(TestCase):
    def setUp(self):
        self.client = MercuryApi(
            {
                "username": "username",
                "password": "password",
                "url": "https://mercury-dev.esthenos.in",
            }
        )

        self.post_api_mock = mock.patch(
            "mercuryclient.api.MercuryApi._post_json_http_request"
        ).start()
        self.addCleanup(self.post_api_mock.stop)
        self.get_api_mock = mock.patch(
            "mercuryclient.api.MercuryApi._get_json_http_request"
        ).start()
        self.addCleanup(self.get_api_mock.stop)

    def test_wrong_id_type_raises_exception(self):
        with self.assertRaises(Exception) as exc:
            self.client.request_verify_id(
                "Something", "ABCDE1234", "aadhaarapi", "some_profile"
            )

        self.assertEqual(str(exc.exception), "Something is not a valid id_type")
        self.post_api_mock.assert_not_called()

    def test_request_id_verification(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)

        self.client.request_verify_id(
            "PAN_CARD", "ABCDE1234", "aadhaarapi", "some_profile"
        )

        self.post_api_mock.assert_called_with(
            "api/v1/id_verification/",
            data={
                "provider": "aadhaarapi",
                "profile": "some_profile",
                "id_type": "PAN_CARD",
                "id_number": "ABCDE1234",
            },
            send_request_id=True,
            add_bearer_token=True,
        )

    def test_request_id_verification_passport(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)

        passport_details = PassportDetails(
            first_name="Sample",
            last_name="Name",
            date_of_birth=date(1991, 1, 1),
            expiry_date=date(1995, 6, 6),
            gender="MALE",
            passport_type="PERSONAL",
            country="INDIA",
        )

        with self.assertRaises(Exception) as exc:
            self.client.request_verify_id(
                "PASSPORT", "ABCDE1234", "aadhaarapi", "some_profile"
            )

        self.assertEqual(
            str(exc.exception), "passport_details required for PASSPORT ID Type"
        )

        self.client.request_verify_id(
            "PASSPORT",
            "ABCDE1234",
            "aadhaarapi",
            "some_profile",
            passport_details=passport_details,
        )

        self.post_api_mock.assert_called_with(
            "api/v1/id_verification/",
            data={
                "provider": "aadhaarapi",
                "profile": "some_profile",
                "id_type": "PASSPORT",
                "id_number": "ABCDE1234",
                "passport_details": {
                    "first_name": "Sample",
                    "last_name": "Name",
                    "date_of_birth": "1991-01-01",
                    "expiry_date": "1995-06-06",
                    "gender": "MALE",
                    "passport_type": "PERSONAL",
                    "country": "INDIA",
                },
            },
            send_request_id=True,
            add_bearer_token=True,
        )

    def test_request_exception_raised_if_status_code_error(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 401
        self.post_api_mock.return_value = ("random_string", mock_response)

        with self.assertRaises(Exception):
            self.client.request_verify_id(
                "PAN_CARD", "ABCDE1234", "aadhaarapi", "some_profile"
            )

    def test_request_api_succeeds_if_status_code_success(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)

        request_id = self.client.request_verify_id(
            "PAN_CARD", "ABCDE1234", "aadhaarapi", "some_profile"
        )
        self.assertEqual(request_id, "random_string")

    def test_response_verify_id(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 200
        self.get_api_mock.return_value = ("random_string", mock_response)
        self.client.get_verify_id_result("random_string")

        self.get_api_mock.assert_called_with(
            "api/v1/id_verification/",
            headers={"X-Mercury-Request-Id": "random_string"},
            send_request_id=False,
            add_bearer_token=True,
        )

    def test_response_exception_raised_if_status_code_error(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 401
        self.get_api_mock.return_value = ("random_string", mock_response)
        with self.assertRaises(Exception):
            self.client.get_verify_id_result("random_string")

    def test_response_api_succeeds_if_status_code_success(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 200
        mock_response.json = mock.MagicMock()
        mock_response.json.return_value = {"status": "SUCCESS"}
        self.get_api_mock.return_value = ("random_string", mock_response)

        request_id, result = self.client.get_verify_id_result("random_string")
        self.assertEqual(request_id, "random_string")
        self.assertEqual(result["status"], "SUCCESS")



import json
from mercuryclient.types.insurance.request import InsuranceRequest


class InsuranceMixin:
    def create_insurance_policy(self, request_obj: InsuranceRequest, profile: str):

        """
        Create Insurance policy using the mercury service

        :param: request_obj: InsuranceRequest
        :param profile: An existing profile in Mercury.
        :return: (request_id, status,plicy data )
        """

        api = "api/v1/insurance/create_insurace/"

        request_dict = json.loads(request_obj.json(exclude_unset=True))
        request_dict["profile"] = profile

        request_id, r = self._post_json_http_request(
            path=api, data=request_dict, send_request_id=True, add_bearer_token=True
        )

        if r.status_code == 201:

            return {"request_id": request_id, "data": r.content}

        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error while sending Insurance Request Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )



from datetime import date
import json
from typing import Tuple

from mercuryclient.types.id_verification.enums import IDTypes
from mercuryclient.types.id_verification.request import PassportDetails


class IDVerificationMixin:
    """
    Mixin for verifying IDs
    """

    def request_verify_id(
        self,
        id_type: IDTypes,
        id_number: str,
        provider: str,
        profile: str,
        passport_details: PassportDetails = None,
        **kwargs,
    ) -> str:
        api = "api/v1/id_verification/"

        try:
            IDTypes(id_type)
        except ValueError:
            raise Exception(f"{id_type} is not a valid id_type")

        data = {
            "provider": provider,
            "profile": profile,
            "id_type": id_type,
            "id_number": id_number,
        }

        if id_type == IDTypes.PASSPORT.value:
            if not passport_details:
                raise Exception("passport_details required for PASSPORT ID Type")

            passport_details_dict = json.loads(
                passport_details.json(exclude_unset=True)
            )
            data["passport_details"] = passport_details_dict

        if id_type == IDTypes.DRIVING_LICENSE.value:
            date_of_birth = kwargs.get("date_of_birth")
            if not date_of_birth:
                raise Exception("date_of_birth required for DRIVING_LICENSE ID Type")
            elif not isinstance(date_of_birth, date):
                raise Exception("Date of Birth should be date-type object")
            data["date_of_birth"] = date_of_birth.strftime("%Y-%m-%d")
        request_id, r = self._post_json_http_request(
            api, data=data, send_request_id=True, add_bearer_token=True
        )

        if r.status_code == 201:
            return request_id

        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error while sending ID verification request. Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )

    def get_verify_id_result(self, request_id: str) -> Tuple[str, dict]:
        api = "api/v1/id_verification/"

        request_id, r = self._get_json_http_request(
            api,
            headers={"X-Mercury-Request-Id": request_id},
            send_request_id=False,
            add_bearer_token=True,
        )

        if r.status_code == 200:
            result = r.json()
            return request_id, result

        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error getting ID Verification result. Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )



from unittest import mock
from unittest import TestCase

from mercuryclient.api import MercuryApi
from mercuryclient.types.bank_account_verify.request import AccountVerifyRequest


class AccountVerifyMixinTest(TestCase):
    def setUp(self):

        self.client = MercuryApi(
            {
                "username": "username",
                "password": "password",
                "url": "https://mercury-dev.esthenos.in",
            }
        )

        self.request_obj = AccountVerifyRequest(
            name="test",
            phone="9874563210",
            bank_account_number="874512369854",
            bank_ifsc_code="ES001",
        )

        self.expected_dict = {
            "name": "test",
            "phone": "9874563210",
            "bank_account_number": "874512369854",
            "bank_ifsc_code": "ES001",
            "profile": "some_profile",
            "provider": "some_provider",
        }

        self.post_api_mock = mock.patch(
            "mercuryclient.api.MercuryApi._post_json_http_request"
        ).start()
        self.addCleanup(self.post_api_mock.stop)

    def test_request_account_verify(self):

        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        self.client.verify_bank_account(
            self.request_obj, "some_provider", "some_profile"
        )
        self.post_api_mock.assert_called_with(
            path="api/v1/account_verify/",
            data=self.expected_dict,
            send_request_id=True,
            add_bearer_token=True,
        )

    def test_request_exception_raised_if_status_code_error(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 401
        self.post_api_mock.return_value = ("random_string", mock_response)
        with self.assertRaises(Exception):
            self.client.verify_bank_account(
                self.request_obj, "some_provider", "some_profile"
            )

    def test_request_api_succeeds_if_status_code_success(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_id = self.client.verify_bank_account(
            self.request_obj, "some_provider", "some_profile"
        )
        self.assertEqual(request_id["request_id"], "random_string")



from datetime import date
from typing import List
from typing import Optional

from pydantic import BaseModel
from pydantic import Field
from pydantic import root_validator
from pydantic import validator

from ..common import Pincode
from ..validators import normalize_str
from .enums import AddressType
from .enums import CreditRequestType
from .enums import Gender
from .enums import IdentityType
from .enums import InquiryPurposeType
from .enums import InquiryPurposeTypeDescription
from .enums import InquiryStage
from .enums import PhoneNumberType
from .enums import RelationType
from .enums import State


class Identity(BaseModel):
    id_type: IdentityType
    id_number: str = Field(max_length=30)

    _normalize_id_num = validator("id_number", allow_reuse=True, pre=True)(
        normalize_str
    )


class Relation(BaseModel):
    relation_type: RelationType
    relation_name: str = Field(max_length=30)

    _normalize_name = validator("relation_name", allow_reuse=True, pre=True)(
        normalize_str
    )


class PhoneNumber(BaseModel):
    number_type: PhoneNumberType
    phone_number: str = Field(max_length=20)

    _normalize_num = validator("phone_number", allow_reuse=True, pre=True)(
        normalize_str
    )


class Address(BaseModel):
    address_type: AddressType
    address_line_1: str = Field(min_length=5, max_length=150)
    city: str = Field(max_length=20)
    state: State
    pincode: Pincode

    _normalize_strs = validator("address_line_1", "city", allow_reuse=True, pre=True)(
        normalize_str
    )


class Applicant(BaseModel):
    name: str = Field(max_length=100, strip_whitespace=True)
    other_names: Optional[List[str]] = Field(max_items=4)
    date_of_birth: date
    gender: Optional[Gender]
    identities: List[Identity] = Field(min_items=1)
    nominee: Optional[Relation]
    key_person: Optional[Relation]
    other_relations: Optional[List[Relation]] = Field(max_items=4)
    phone_numbers: Optional[List[PhoneNumber]]
    addresses: List[Address] = Field(min_items=1)

    _normalize_strs = validator("name", allow_reuse=True, pre=True)(normalize_str)
    _normalize_str_list = validator(
        "other_names", allow_reuse=True, pre=True, each_item=True
    )(normalize_str)

    @root_validator
    def check_min_one_relation(cls, values: dict):
        relations = list()
        relations.append(values.get("nominee"))
        relations.append(values.get("key_person"))
        relations.extend(values.get("other_relations", []) or [])
        assert any(
            relations
        ), "at least one of the following is required: nominee, key_person, other_relations"

        return values


class HighmarkRequest(BaseModel):
    inquiry_reference_number: str = Field(max_length=50)
    credit_request_type: CreditRequestType
    credit_transaction_id: str = Field(max_length=50)
    inquiry_purpose_type: InquiryPurposeType
    inquiry_purpose_type_desc: InquiryPurposeTypeDescription
    inquiry_stage: InquiryStage
    kendra_id: Optional[str] = Field(max_length=30)
    branch_id: Optional[str] = Field(max_length=30)
    los_app_id: str = Field(max_length=50)
    loan_amount: Optional[int]
    applicant: Applicant

    _normalize_strs = validator(
        "inquiry_reference_number",
        "credit_transaction_id",
        "los_app_id",
        allow_reuse=True,
        pre=True,
    )(normalize_str)



from os import path

from setuptools import find_packages
from setuptools import setup

this_directory = path.abspath(path.dirname(__file__))

with open(path.join(this_directory, "README.rst")) as f:
    long_description = f.read()

requirements = ["requests >=2.25.0,<3", "PyJWT >=1.7.0,<2", "pydantic >=1.7.0,<2"]
dev_requirements = ["pre-commit", "wheel", "twine"]

setup(
    name="mercuryclient",
    version="1.15.1",
    description="Python SDK for Mercury service",
    long_description=long_description,
    long_description_content_type="text/x-rst",
    url="https://bitbucket.org/esthenos/mercury",
    author="Esthenos Technologies Private Limited",
    author_email="atul@esthenos.com ",
    license="Proprietary License",
    packages=find_packages(),
    install_requires=requirements,
    extras_require={"dev": dev_requirements},
    python_requires=">=3.6",
    zip_safe=False,
)



def normalize_str(value: str) -> str:
    """
    Method to normalize strings. Raises value error if after stripping and removing
    newlines, string is empty

    :param value: String
    :type value: str
    :return: Normalized string
    :rtype: str
    """
    # Remove leading and trailing spaces
    value = value.strip()
    # Remove newlines
    value = "".join(value.splitlines())

    if not value:
        raise ValueError("string cannot be empty")

    return value



class MailMixin:
    """
    Mixin for sending emails
    """

    def send_mail(self, recipients, subject, body, provider, profile):
        """
        Send email using the mercury service

        :param recipients: List of recipients
        :param subject: Subject of the email
        :param body: Body of the email.
        :param provider: Either 'ses' or 'mailgun'
        :param profile: An existing profile in Mercury. The profile has to correspond
        to the provider.
        :return: (request_id, status)
        """
        api = "api/v1/mail/"

        data = {
            "provider": provider,
            "profile": profile,
            "subject": subject,
            "recipients": recipients,
            "message": body,
        }

        request_id, r = self._post_json_http_request(
            api, data=data, send_request_id=True, add_bearer_token=True
        )

        if r.status_code == 201:
            return {"request_id": request_id, "status": "Success"}

        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error while sending mail. Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )



from unittest import mock
from unittest import TestCase

from mercuryclient.api import MercuryApi
from mercuryclient.types.insurance.request import (
    AdditionalInfo,
    InsuranceRequest,
    LoanData,
    PersonData,
    Premium,
    Customer,
    Insured,
    AccountData,
    Nominee,
)


class InsuranceMixinTest(TestCase):
    def setUp(self):
        self.client = MercuryApi(
            {
                "username": "username",
                "password": "password",
                "url": "https://mercury-dev.esthenos.in",
            }
        )
        self.request_obj = InsuranceRequest(
            plan="esthenos_creditshield",
            partner_id="esthenos",
            plan_type=2,
            reference_id="jsdjvbsjdvl001",
            category="loan",
            premium=Premium(amount=1180),
            customer=Customer(
                id="CUST001",
                name="Bittu",
                phone="9646432812",
                email="bittu.kumar_testi_blr@acko.tech",
                state="Karnataka",
            ),
            loan=LoanData(
                amount=100000,
                provider_id=2,
                provider_name="Lorem Ipsum",
                start_date="2022-05-23T12:22:27.873593Z",
                end_date="2025-03-28T12:22:27.873593Z",
                emi_amount=1200,
                hospicash_amount=1000,
                disbursement_date="2022-05-22T12:22:27.873593Z",
                account_number="ACC01",
                tenure_in_months=34,
                type="car_loan",
                family_type="1A",
                application_number="ACC01",
                person=[
                    PersonData(
                        insured=Insured(
                            name="Bittu",
                            phone="9646432812",
                            email="bittu.kumar_tesi_blr@acko.tech",
                            pincode="111111",
                            city="sample_city",
                            address="sample_address",
                            gender="Male",
                            occupation="salaried",
                            age=27,
                            dob="1995-02-05",
                            account=AccountData(
                                number="9646432812",
                                ifsc="HDFC0123456",
                                category="Regular",
                            ),
                        ),
                        nominee=Nominee(
                            name="Temp",
                            relationship="mother",
                            phone="9988509343",
                            dob="1991-08-14",
                        ),
                    )
                ],
            ),
            additional_info=AdditionalInfo(
                declaration_health="Y", assignment_clause="Y", collection_status="Y"
            ),
        )

        self.request_dict = {
            "profile": "some_profile",
            "plan": "esthenos_creditshield",
            "partner_id": "esthenos",
            "plan_type": 2,
            "reference_id": "jsdjvbsjdvl001",
            "category": "loan",
            "premium": {"amount": 1180},
            "customer": {
                "id": "CUST001",
                "name": "Bittu",
                "phone": "9646432812",
                "email": "bittu.kumar_testi_blr@acko.tech",
                "state": "Karnataka",
            },
            "loan": {
                "amount": 100000,
                "provider_id": 2,
                "provider_name": "Lorem Ipsum",
                "start_date": "2022-05-23T12:22:27.873593Z",
                "end_date": "2025-03-28T12:22:27.873593Z",
                "emi_amount": 1200,
                "hospicash_amount": 1000,
                "disbursement_date": "2022-05-22T12:22:27.873593Z",
                "account_number": "ACC01",
                "tenure_in_months": 34,
                "type": "car_loan",
                "family_type": "1A",
                "application_number": "ACC01",
                "person": [
                    {
                        "insured": {
                            "name": "Bittu",
                            "phone": "9646432812",
                            "email": "bittu.kumar_tesi_blr@acko.tech",
                            "pincode": "111111",
                            "city": "sample_city",
                            "address": "sample_address",
                            "gender": "Male",
                            "occupation": "salaried",
                            "age": 27,
                            "dob": "1995-02-05",
                            "account": {
                                "number": "9646432812",
                                "ifsc": "HDFC0123456",
                                "category": "Regular",
                            },
                        },
                        "nominee": {
                            "name": "Temp",
                            "relationship": "mother",
                            "phone": "9988509343",
                            "dob": "1991-08-14",
                        },
                    }
                ],
            },
            "additional_info": {
                "declaration_health": "Y",
                "assignment_clause": "Y",
                "collection_status": "Y",
            },
        }
        self.post_api_mock = mock.patch(
            "mercuryclient.api.MercuryApi._post_json_http_request"
        ).start()
        self.addCleanup(self.post_api_mock.stop)

    def test_request_insurance_policy(self):

        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)

        self.client.create_insurance_policy(self.request_obj, "some_profile")

        self.post_api_mock.assert_called_with(
            path="api/v1/insurance/create_insurace/",
            data=self.request_dict,
            send_request_id=True,
            add_bearer_token=True,
        )

    def test_request_exception_raised_if_status_code_error(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 401
        self.post_api_mock.return_value = ("random_string", mock_response)

        with self.assertRaises(Exception):
            self.client.create_insurance_policy(self.request_obj, "some_profile")

    def test_request_api_succeeds_if_status_code_success(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)

        request_id = self.client.create_insurance_policy(
            self.request_obj, "some_profile"
        )
        self.assertEqual(request_id["request_id"], "random_string")
